@@ -14,7 +14,7 @@ import pprint
 import re
 import sys
 
-from .dprint import dprint
+from pyxllib.basic.dprint import dprint
 
 
 def strfind(fullstr, objstr, *, start=None, times=0, overlap=False):
@@ -138,7 +138,8 @@ def natural_sort_key(key):
 
 
 def natural_sort(ls, only_use_digits=False):
-    """自然排序
+    """ 自然排序
+
     :param only_use_digits: 正常会用数字作为分隔，切割每一部分进行比较
         如果只想比较数值部分，可以only_use_digits=True
     """
@@ -151,7 +152,8 @@ def natural_sort(ls, only_use_digits=False):
 
 
 def typename(c):
-    """简化输出的type类型
+    """ 简化输出的type类型
+
     >>> typename(123)
     'int'
     """
@@ -688,3 +690,63 @@ def digit2weektag(d):
         return '周' + '一二三四五六日'[d - 1]
     else:
         raise ValueError
+
+
+def fullwidth2halfwidth(ustring):
+    """ 把字符串全角转半角
+
+    python3环境下的全角与半角转换代码和测试_大数据挖掘SparkExpert的博客-CSDN博客:
+    https://blog.csdn.net/sparkexpert/article/details/82749207
+
+    >>> fullwidth2halfwidth("你好ｐｙｔｈｏｎａｂｄａｌｄｕｉｚｘｃｖｂｎｍ")
+    '你好pythonabdalduizxcvbnm'
+    """
+    ss = []
+    for s in ustring:
+        for uchar in s:
+            inside_code = ord(uchar)
+            if inside_code == 12288:  # 全角空格直接转换
+                inside_code = 32
+            elif 65281 <= inside_code <= 65374:  # 全角字符（除空格）根据关系转化
+                inside_code -= 65248
+            ss.append(chr(inside_code))
+    return ''.join(ss)
+
+
+def fullwidth2halfwidth2(ustring):
+    """ 不处理标点符号的版本
+
+    >>> fullwidth2halfwidth2("你好ｐｙｔｈｏｎａｂｄａ，ｌｄｕｉｚｘｃｖｂｎｍ")
+    '你好pythonabda，lduizxcvbnm'
+    """
+    ss = []
+    for s in ustring:
+        for uchar in s:
+            if uchar in '：；！（），？＂．':
+                ss.append(uchar)
+            else:
+                inside_code = ord(uchar)
+                if inside_code == 12288:  # 全角空格直接转换
+                    inside_code = 32
+                elif 65281 <= inside_code <= 65374:  # 全角字符（除空格）根据关系转化
+                    inside_code -= 65248
+                ss.append(chr(inside_code))
+    return ''.join(ss)
+
+
+def halfwidth2fullwidth(ustring):
+    """ 把字符串全角转半角
+
+    >>> halfwidth2fullwidth("你好ｐｙｔｈｏｎａｂｄａｌｄｕｉｚｘｃｖｂｎｍ")
+    '你好ｐｙｔｈｏｎａｂｄａｌｄｕｉｚｘｃｖｂｎｍ'
+    """
+    ss = []
+    for s in ustring:
+        for uchar in s:
+            inside_code = ord(uchar)
+            if inside_code == 32:  # 全角空格直接转换
+                inside_code = 12288
+            elif 33 <= inside_code <= 126:  # 全角字符（除空格）根据关系转化
+                inside_code += 65248
+            ss.append(chr(inside_code))
+    return ''.join(ss)
